@@ -367,13 +367,34 @@ sugar_style_draw_box (GtkStyle       *style,
             sugar_draw_button_default (cr, &info);
         else
             sugar_draw_button (cr, &info);
+    } else if (DETAIL ("entry-progress")) {
+        SugarInfo info;
 
+        sugar_fill_generic_info (&info, style, state_type, shadow_type, widget, detail, x, y, width, height);
+
+        /* Corner detection. */
+        if (HINT ("comboboxentry") || HINT("spinbutton")) {
+            info.cont_edges = info.ltr ? EDGE_RIGHT : EDGE_LEFT;
+            sugar_remove_corners (&info.corners, info.cont_edges);
+
+            /* Remove the padding on one side. */
+            width += info.rc_style->thick_line_width;
+            if (!info.ltr) {
+                x -= info.rc_style->thick_line_width;
+            }
+        }
+
+        sugar_draw_entry_progress (cr, &info);
     } else if (DETAIL ("spinbutton")) {
         SugarInfo info;
         sugar_fill_generic_info (&info, style, state_type, shadow_type, widget, detail, x, y, width, height);
 
-        /* Fill the background with bg_color. */
-        sugar_fill_background (cr, &info);
+        /* Fill the background as it is initilized to base[NORMAL].
+         * Relevant GTK+ bug: http://bugzilla.gnome.org/show_bug.cgi?id=513471
+         * The fill only happens if no hint has been added by some application
+         * that is faking GTK+ widgets. */
+        if (!widget || !g_object_get_data(G_OBJECT (widget), "transparent-bg-hint"))
+            sugar_fill_background (cr, &info);
 
         info.cont_edges = info.ltr ? EDGE_LEFT : EDGE_RIGHT;
         sugar_remove_corners (&info.corners, info.cont_edges);
@@ -423,11 +444,12 @@ sugar_style_draw_box (GtkStyle       *style,
 
             sugar_fill_generic_info (&info, style, state_type, shadow_type, widget, detail, x, y, width, height);
             
-            if (widget && !GTK_WIDGET_IS_SENSITIVE (widget))
+            if (widget && !gtk_widget_is_sensitive (widget))
                 info.state = GTK_STATE_INSENSITIVE;
 
             /* Needed because the trough and bar are cached in a buffer inside GtkProgress. */
-            sugar_fill_background (cr, &info);
+            if (!widget || !g_object_get_data(G_OBJECT (widget), "transparent-bg-hint"))
+                sugar_fill_background (cr, &info);
             sugar_draw_progressbar_trough (cr, &info);
         }
     } else if (DETAIL ("bar")) {
@@ -436,7 +458,7 @@ sugar_style_draw_box (GtkStyle       *style,
 
             sugar_fill_generic_info (&info, style, state_type, shadow_type, widget, detail, x, y, width, height);
             
-            if (widget && !GTK_WIDGET_IS_SENSITIVE (widget))
+            if (widget && !gtk_widget_is_sensitive (widget))
                 info.state = GTK_STATE_INSENSITIVE;
             
             if (widget && GTK_IS_PROGRESS_BAR (widget)) {
@@ -619,12 +641,13 @@ sugar_style_draw_shadow (GtkStyle       *style,
         if (widget && GTK_WIDGET_HAS_FOCUS (widget)) {
             info.state = GTK_STATE_ACTIVE;
         }
-        if (widget && !GTK_WIDGET_IS_SENSITIVE (widget)) {
+        if (widget && !gtk_widget_is_sensitive (widget)) {
             info.state = GTK_STATE_INSENSITIVE;
         }
 
         /* Fill the background with bg_color. */
-        sugar_fill_background (cr, &info);
+        if (!widget || !g_object_get_data(G_OBJECT (widget), "transparent-bg-hint"))
+            sugar_fill_background (cr, &info);
         sugar_draw_entry (cr, &info);
     } else {
         gdouble line_width;
